import os


def next_numerical_file_name(folder_path, base=''):
    """

    CHANGELOG

    Added 19.11.2018

    @param str folder_path:
    @param str base:
    @return:
    """
    if os.path.exists(folder_path) and os.path.isdir(folder_path):

        # Iterating through all the elements of the folder and looking only at the file names.
        # We try to convert a file name to an integer to find out if it is a numerical file name.
        # We make a list of all the numbers, that are already used and then return the next bigger
        # number, that can still be used.

        already_used_numbers = []
        for root, dirs, files in os.walk(folder_path):

            # only looking at the files:
            for file_name in files:  # type: str
                try:
                    # Getting the pure name of the file, WITHOUT the file type extension.
                    # Also removing the base name string, that is either prefix or suffix
                    file_name_pure = file_name.split('.')[0].replace(base, '')
                    number = int(file_name_pure)
                    already_used_numbers.append(number)
                finally:
                    pass

        # At the end we retrieve the maximum used number and return the numerically next one
        max_already_used_number = max(already_used_numbers)
        return max_already_used_number + 1

    else:
        raise Exception('The given path {} is not a valid folder for file enumeration !'.format(folder_path))



import numpy as np

import copy


class LightningImage:
    """
    A numpy array wrapper object for manipulating grayscale images.

    CHANGELOG

    Added 04.11.2018

    @author Jonas Teufel
    """

    def __init__(self, img):
        """
        The constructor.

        CHANGELOG

        Added 04.11.2018

        Changed 06.11.2018
        Added separate attributes for the height and width of the image

        Changed 13.11.2018
        Now a LightningImage object can be passed to the constructor, which will result in the
        creation of a copy of that image.
        Added a original field

        @param np.ndarray img: The array should be two dimensions, which means only grayscale images
        """
        if isinstance(img, LightningImage):
            # If the input was another lightning image object it gets copied, which means
            # the original also gets copied
            self.array = copy.deepcopy(img.array)
            #self.original = copy.deepcopy(img.original)
        else:
            self.array = copy.deepcopy(img)
            #self.original = copy.deepcopy(img)

        # 06.11.2018
        # Saving the height and the width of the image and thus the dimensions of the array as well
        self.width = self.array.shape[1]
        self.height = self.array.shape[0]

    def copy(self):
        """
        Returns a copy of the image object

        CHANGELOG

        Added 19.11.2018

        @return: LightningImage
        """
        return LightningImage(copy.deepcopy(self.array))

    def get_mask(self, threshold=128):
        """
        This method will return an array object with the same shape as the image. Based on a
        threshold value this array will contain a 1 if the image value exceeds the threshold at that
        index and a 0 otherwise, thus forming a "boolean" image mask

        CHANGELOG

        Added 19.11.2018

        @param threshold:
        @return:
        """
        # Creating a function, which will return 1 if the value of the element ist greater than the
        # threshold and 0 otherwise.
        # This function will be applied as a element wise transformation, thus creating a mask array
        def threshold_mask(value, i, j):
            if value > threshold:
                return 1
            else:
                return 0

        # The transformation is applied on a copy of image, so that the creation of the mask doesnt
        # influence the array.
        cpy = self.copy()
        cpy.transform_element_wise(threshold_mask)
        return cpy.array

    def transform_masked(self, f, mask, replace=None):
        """
        Given a function and a mask, the function will be applied to each element, where the mask evaluates to True.

        CHANGELOG

        Added 16.11.2018

        @param f:           The function to be applied to each element. has to return a float value between 0 and 255.
                            Has to accept 3 arguments: the old element value, axis0 index, axis1 index
        @param mask:        An array, that has exactly the same dimensions as the image to transform. Contains only
                            0 and 1 (True and False). The given function will only be applied to elements at indices,
                            where the mask element evaluates to True.
        @param replace:     Optionally an int in the range between 0 and 255. Every pixel, that is not being masked will
                            be replaced with this constant value in the transformed image.
                            If it is None, the new image will have the same value as the old image in a unmasked element
                            DEFAULT is None.
        @return:
        """
        new = np.zeros(self.array.shape, np.uint8)
        it = np.nditer(self.array, flags=['multi_index'])
        it_new = np.nditer(new, op_flags=['writeonly'])
        it_mask = np.nditer(mask)
        while not it.finished:
            # Only doing anything if the current pixel is masked
            if it_mask[0]:
                # Retrieving the value for the transformed array as the transformation of the current array element
                it_new[0] = f(it[0], it.multi_index[0], it.multi_index[1])

            else:
                # In case there is a replace value given. All other elements, that are not masked are replaced by this
                # constant value
                if replace is not None:
                    it_new[0] = replace
                # Otherwise the value from the original matrix will be kept
                else:
                    it_new[0] = it[0]

            # Iterating to the next element
            it.iternext()
            it_new.iternext()
            it_mask.iternext()

        # Switching to the new array
        self.array = new

    def transform_element_wise(self, f):
        """
        Given a function, this function will be applied to each element of the matrix.

        CHANGELOG

        Added 16.11.2018

        @param f:   The function to be applied to the elements. Has to return a 8 bit integer. Has to accept 3
                    arguments: The old element value, the axis0 index, the axis1 index
        @return:
        """
        # Creating a new empty matrix with the same dimensions
        new = np.zeros(self.array.shape, np.uint8)
        it = np.nditer(self.array, flags=['multi_index'])
        it_new = np.nditer(new, op_flags=['writeonly'])
        while not it.finished:
            # Retrieving the value for the transformed array as the transformation of the current array element
            value = f(it[0], it.multi_index[0], it.multi_index[1])
            # Setting the element of the new matrix
            it_new[0] = value

            # Incrementing the iterators
            it.iternext()
            it_new.iternext()

        # Switching to the new array
        self.array = new

    def transform(self, f):
        """
        Takes a function that transforms a whole matrix and applies it on the image.

        CHANGELOG

        Added 16.11.2918

        @param f:   The function to be used on the matrix. Has to accept 1 argument being the whole array, that
                    describes the image. has to return an array object with the same dimensions and data type.
        @return:
        """
        # The function creates a new two dimensional array.
        # Creating a new Lightning image object from the transformation result
        self.array = f(self.array)

    def darken(self, threshold, replace=0):
        """
        This method will select all the pixels with the grayscale value LOWER than the given threshold and replace them
        with the given fixed value 'replace'. The default to replace with is 0(dark), thus darkening the pixels

        CHANGELOG

        Added 04.11.2018

        @param int threshold:   All the pixels, that are smaller than this will be replaced. This has to be 8 bit int
        @param int replace:     The fixed value to replace with
        @return: void
        """
        it = np.nditer(self.array, flags=['multi_index'], op_flags=['writeonly'])
        while not it.finished:
            if it[0] <= threshold:
                it[0] = replace
            it.iternext()

    def lighten(self, threshold, replace=255):
        """
        This method will select all the pixels with a grayscale value BIGGER than the given threshold and replace them
        with the given fixed value 'replace'. The default to replace with is 255(white) thus lightening those pixels up

        CHANGELOG

        Added 04.11.2018

        @param int threshold:   All values, that are bigger than this will be replaced. This has to be in the range 0
                                to 255
        @param int replace:     The fixed value to replace all the pixels with, that are bigger than the threshold
        @return: void
        """
        it = np.nditer(self.array, flags=['multi_index'], op_flags=['writeonly'])
        while not it.finished:
            if it[0] >= threshold:
                it[0] = replace
            it.iternext()

    def invert(self):
        """
        Inverts all the grayscale values of all the pixels of the picture

        CHANGELOG

        Added 04.11.2018

        @return: void
        """
        it = np.nditer(self.array, flags=['multi_index'], op_flags=['writeonly'])
        while not it.finished:
            it[0] = 255 - it[0]
            it.iternext()

    def difference(self, other, threshold=10, replace=255, invert=False, ):
        """
        This method will essentially substract two images with one another. But it will do so in a commutative way,
        which means that for each pixel the absolute difference is being calculated, which prevents negative values,
        but also means, that the operation does not have an order.
        Additionally all resulting differences lower than a 'threshold' can be replaced by a fixed values 'replace'.
        This is to be able to remove noise from the picture, as small differences in two different pictures of the
        same thing are most likely to be random noise.


        why not just a plain & normal subtraction?
        Because the numpy arrays, which represent images use unsigned 8 bit integers and if the result of one pixel
        subtraction will be negative, this will cause the data type to start at the other end again instead.
        Example: 0 - 1 = 254

        CHANGELOG

        Added 04.11.2018

        @param LightningImage other:    The other image, which is supposed to be subtracted from this one
        @param int threshold:           Every resulting difference value below this given int will be replaced with a
                                        fixed value. If this is 0 no replacements will be made
        @param int replace:             The value to replace pixels with, that are below the threshold
        @param bool invert:
        @return LightningImage:         The new image
        """
        new = np.zeros(self.array.shape)
        it = np.nditer(new, flags=['multi_index'], op_flags=['writeonly'])
        while not it.finished:
            value_self = self.array[it.multi_index]
            value_other = other.array[it.multi_index]
            diff = abs(int(value_self) - int(value_other))

            if diff < threshold:
                it[0] = replace
            else:
                it[0] = abs(int(value_self) - int(value_other))
                if invert:
                    it[0] = 255 - it[0]

            it.iternext()

        return LightningImage(new)

    def column_sum(self, scale=None):
        """
        Calculates an array, that contains the float sum's of all the elements of each column. he resulting array can
        optionally be scaled relative to a new peak value. The resulting array has as many elements as the image's
        width

        CHANGELOG

        Added 06.11.2018

        @param scale:   The new peak value to scale to. DEFAULT is None, which means no scaling will be done
        @return:
        """
        # This is just a special case of the general case 'directional_sum', which is a implementation for both
        # directions
        return self.directional_sum(0, scale)

    def row_sum(self, scale=None):
        """
        Calculates an array, that contains the float sum's of all the elements of each row. The resulting array can
        optionally be scaled relative to a new peak value. The resulting array has as many elements as the image's
        height

        CHANGELOG

        Added 06.11.2018

        @param scale:   The new peak value to scale to. DEFAULT is None, which means no scaling will be done
        @return:
        """
        # This is just a special case of the general case 'directional_sum', which is a implementation for both
        # directions
        return self.directional_sum(1, scale)

    def directional_sum(self, axis, scale):
        """
        Calculates an array, that contains the float sum's of each element either the rows or the columns. The array
        thus has the same size as either the column or the row of the image.
        These sum values could possibly take any value from 0 to 255 * array length (for an all white picture). The
        scale parameter optionally limits the maximum value possible by scaling all elements down proportionally.

        CHANGELOG

        Added 06.11.2018

        @param int axis:    Either 0 or 1. A 1 would be the sum of all rows and 0 the sum of all columns
        @param int scale:   An integer, which sets the maximum value for the sums. All values get scaled
                            relative to this value.
        @return: An array of either the height or width of the image
        """

        # Creating a new one dimensional array with just as many elements as there are pixel rows in the image, which
        # will contain the sum values of each row.
        # For a two dimensional array (as the image is one), the second item of the shape attribute will contain the
        # length of a column and thus the amount of rows
        sum_array = np.ndarray(self.array.shape[axis], np.float64)

        # Iterating through all the rows of the array ans summing up the individual elements
        for i in range(self.array.shape[axis]):
            temp_sum = 0
            for j in range(self.array.shape[1 - axis]):
                # Numpy arrays support multi indexing in a single bracket for higher dimensions and this is also the
                # more efficient way
                if axis:
                    temp_sum += self.array[j, i]
                else:
                    temp_sum += self.array[i, j]

            sum_array[i] = temp_sum

        # If there is a scale value given, all the values in the array will be scaled, so that the maximum value is
        # at most the scale value
        if scale is not None:
            maximum = max(sum_array)
            # The np.vectorize function is used to convert a normal python function object into a function, that can
            # be applied on a numpy array, element-wise
            scaling = np.vectorize(lambda x: (x / maximum) * scale, otypes=[np.float64])
            sum_array = scaling(sum_array)

        return sum_array

    def __sub__(self, other):
        """
        The subtraction operation.

        This creates a new image object, where each pixel of the resulting image has the absolute difference of the
        two initial pictures as the grayscale value.

        CHANGELOG

        Added 06.11.2018

        @param LightningImage other:   The image to be subtracted
        @return: LightningImage
        """
        # The pure subtraction is just a special case of the general difference function, but without the whole noise
        # replacement function
        return self.difference(other, threshold=0)


# ALL THE FUNCTIONS THAT CAN BE APPLIED ON THE IMAGE OBJECT
# IMAGE TRANSFORMATIONS




import matplotlib.pyplot as plt
import matplotlib.patches as patches


from lightnimage.image import LightningImage
from lightnimage.engine import *

import os


def plot_lightning_detection_overview(image, ref_image):
    """
    CHANGELOG

    Added 16.11.2018

    @deprecated

    @param LightningImage image:
    @param LightningImage ref_image:
    @return:
    """
    # Creating the sub plots
    f, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2, sharex='col', sharey='row')

    # Calculating a simple subtraction of the two images
    print('Calculating the difference of the pictures')
    subtraction = ref_image - image

    # Plotting that into the first sub plot
    ax1.imshow(subtraction.array, cmap='gray')
    ax1.set_title('difference of the images')

    # Calculating the difference of the two images
    difference = ref_image.difference(image, threshold=70, replace=0)

    # Refining the difference picture to gain a pure mask of the lightning
    print('Processing the difference to get the lightning mask')
    difference.lighten(30, replace=255)

    # Plotting the mask into a subplot
    ax2.imshow(difference.array, cmap='gray')
    ax2.set_title('lightning mask')

    # Computing the areas in which the lightnings are
    print('Calculating the areas, that contain lightning')
    config = {

    }
    area_engine = SimpleAreaSegmentationEngine(config)
    areas = area_engine(difference)

    # Plotting the original picture into the third plot and the calculated areas as red boxed around the lightnings
    ax3.imshow(image.array, cmap='gray')
    ax3.set_title('lightning detection')

    print('Overlaying the detected areas with the original picture')
    for area in areas:
        start = (area[0][0], area[1][0])
        width = area[0][1] - area[0][0]
        height = area[1][1] - area[1][0]
        rect = patches.Rectangle(start, width, height, linewidth=1, edgecolor='r', facecolor='none')
        ax3.add_patch(rect)

    # Removing the lightning from the difference picture by using the mask
    print('Removing the lightning from the difference picture using the lightning mask')
    removed = subtraction.copy()
    removed.transform_masked(lambda v, i, j: 0, difference.get_mask())

    ax4.imshow(removed.array, cmap='gray')
    ax4.set_title('without lightning')

    plt.show()


def plot_simple_lightning_detection(image, ref_image, save_path=None, file_name=None, is_showing=True):
    """
    This function will plot the original pictures and the areas, where the lightnings were detected as
    an overlay of red boxes. Besides the red boxes will be the label, of what the program guesses the
    lightning type to be:
    - "ground": Cloud to ground
    - "cross": Cloud to cloud
    - "???": Couldnt make a guess

    The function will return a list of tuples, where the first element is the string containing the
    type of the lightning and the second being the area tuple describing the area, where the lightning
    can be found.

    CHANGELOG

    Added 19.11.2018

    Changed 19.11.2018
    Fixed bug, where the plot image couldnt be saved

    @param LightningImage image:
    @param LightningImage ref_image:
    @param str save_path:
    @param str file_name:
    @param bool is_showing:
    @return: List(Tuple(str, Tuple(Tuple(int, int), Tuple(int, int))))
    """
    # Creating the sub plots
    f, ax = plt.subplots(1, 1, sharex='col', sharey='row')

    # Calculating a simple subtraction of the two images
    print('Calculating the difference of the pictures')

    # Calculating the difference of the two images
    difference = ref_image.difference(image, threshold=70, replace=0)

    # Refining the difference picture to gain a pure mask of the lightning
    print('Processing the difference to get the lightning mask')
    difference.lighten(30, replace=255)

    # Computing the areas in which the lightnings are
    print('Calculating the areas, that contain lightning')
    config = {

    }
    area_engine = SimpleAreaSegmentationEngine(config)
    areas = area_engine(difference)

    # Filtering out the area that occurs due to the timestamp in the lower right corner
    areas = filter(lambda x: x[0][0] <= image.width * 0.9 or x[1][0] <= image.height * 0.9, areas)

    ax.imshow(image.array, cmap='gray')

    # For each area making a guess if it is a cross or ground lightning
    # depending on whether the area is rather vertical or horizontal
    print('Overlaying the detected areas with the original picture')
    area_types = []
    for area in areas:
        print('Found area {}'.format(str(area)))
        start = (area[0][0], area[1][0])
        width = area[0][1] - area[0][0]
        height = area[1][1] - area[1][0]
        rect = patches.Rectangle(start, width, height, linewidth=1, edgecolor='r', facecolor='none')
        ax.add_patch(rect)

        # Making the guess
        if width >= 1.3 * height:
            guess = 'cross'
        elif height >= 1.3 * width:
            guess = 'ground'
        else:
            guess = '???'

        # Adding the guessed type and the area, which was guessed for to the list
        # which will be returned at the end
        area_types.append((guess, area))

        # plotting the text
        ax.text(start[0], start[1] - 14, guess, size=7, color='r')

    if save_path is not None:

        # In case a file name is given it will be used
        if file_name is not None:
            # 19.11.2018
            # Changed the image type to SVG and changed the method for creating the path from simple
            # string manipulation to path.join function
            file_name_extended = "{}.svg".format(file_name)
            file_path = os.path.join(save_path, file_name_extended)
            print('Saving as "{}"'.format(file_path))
            plt.savefig(file_path, dpi=600)

    # 19.11.2018
    # Added a flag as parameter, whith which the actual display of the plot can be toggled
    if is_showing:
        plt.show()


# 16.11.2018
# Standard library
# Python 2 compatibility for the print function syntax
from __future__ import print_function
import math
from collections import defaultdict

# third party
import numpy as np
from pprint import pprint

# local package
from lightnimage.image import LightningImage
from lightnimage.calculate import *


# ABSTRACT BASE CLASSES #


class AbstractVectorisationEngine:
    """

    CHANGELOG

    Added 05.11.2018

    """

    def __init__(self, lightning_image):
        """

        CHANGELOG

        Added 05.11.2018

        @param LightningImage lightning_image:
        """
        self.lightning_image = lightning_image
        self.array = self.lightning_image.array

    def __call__(self, *args, **kwargs):
        """

        CHANGELOG

        Added 05.11.2018

        @param args:
        @param kwargs:
        @return:
        """
        raise NotImplementedError()


class AbstractAreaSegmentationEngine:

    def __init__(self, config):
        pass

    def __call__(self, lightning_image):
        """
        The engine gets called on a single lightning image object and returns a list of dicts, each containing a value
        for a x, y, length and width keys, defining a rectangle, where the procedure guesses a lightning can be found

        CHANGELOG

        Added 13.11.2018

        @param lightning_image:
        @return:
        """
        raise NotImplementedError()


# IMPLEMENTATIONS #

class SimpleAreaSegmentationEngine(AbstractAreaSegmentationEngine):
    """
    This procedure assumes the image is already processed.

    Given an Image object, this algorithm will calculate the pixel sums along the rows and the columns of the picture.
    The resulting intensity function is then segmented into sub sequences along the axis's by watching, when the
    function first exceeds a certain threshold and when it drops below it again.
    From these sequences along the axis all possible combinations(=areas on the picture) are computed. These areas are
    suspected to contain intensity maximums. In the last step the average of each area is checked against another
    threshold to remove false positives from the final result.

    CHANGELOG

    Added 16.11.2018
    """
    DEFAULT_CONFIG = {
        'threshold':        1.0,
        'checking':         True,
        'check_threshold':  0.03,
    }

    def __init__(self, config):
        """
        The constructor.

        The config dict can have the following parameters:
        - threshold:        A float, which defines the factor between a value of the axis sum function and the global
                            average of the latter that qualifies a value as surpassing the threshold.
                            A threshold RELATIVE to the average of the function. DEFAULT is 1.0
        - checking:         boolean flag of whether or not to perform the last checking operation
        - check_threshold:  A float threshold value relative to 255 the average of an area has to have to qualify as a
                            valid solution to the problem. DEFAULT is 0.03, equates to roughly a grayscale value of 7,
                            which has to be the average of an area.

        CHANGELOG

        Added 16.11.2018

        @param dict config:
        """
        AbstractAreaSegmentationEngine.__init__(self, config)
        self.config = self.DEFAULT_CONFIG.copy()
        self.config.update(config)

        self.current = None

    def __call__(self, lightning_image):
        """
        The main function executed, when the engine is called.

        CHANGELOG

        Added 16.11.2018

        @param LightningImage lightning_image:
        @return: List(Tuple())
        """
        # A copy of the image object is being made, so transformations can be used without disturbing the original
        # image
        self.current = LightningImage(lightning_image)

        # Calculating the row and column sums of the grayscale values
        self.x_sums = self.current.row_sum()
        self.y_sums = self.current.column_sum()

        # Calculating the mean value of both these functions
        x_average = np.average(self.x_sums)
        y_average = np.average(self.y_sums)

        # Calculating the thresholds based on the factor given by the config
        self.x_threshold = x_average * self.config['threshold']
        self.y_threshold = y_average * self.config['threshold']

        # Getting all the possible areas
        x_sequences = threshold_sequencing(self.x_sums, self.config['threshold'])
        y_sequences = threshold_sequencing(self.y_sums, self.config['threshold'])

        areas = combinations_2d(x_sequences, y_sequences)

        if self.config['checking']:
            # Creating areas only from all the possible combinations of two axis's sub sequences also creates a lot
            # of false areas.
            # Here we go through all the areas and essentially compute the average amount of signal within them. Areas
            # are only part of the final solution, if the average within them surpasses a certain threshold
            result = []
            for area in areas:
                # Calculating the average within these areas and only using these that contain a high enough value
                av = average_2d(self.current.array, area)
                # print("Checking {} with average {}".format(str(area), av))
                if (av / 255) >= self.config['check_threshold']:
                    result.append(area)
            return result

        return areas


class SimpleAreaGroupingEngine:

    DEFAULT_CONFIG = {
        'weight_function': lambda d, s: d * s,
        'threshold':       10**4
    }

    def __init__(self, config):
        """
        The constructor

        The config dict can have the following parameters:
        - weight_function:  This is a callable object, which will expect two parameters, the first one being the
                            the distance between two areas and the second one being their combined size (added up).
                            The function has to return a value which will then be compared to the given threshold
        - threshold:        The value which will be compared with the result of the weight function, that has been
                            computed from the pair of two areas.
                            Is the weight smalled than the threshold, the two areas are grouped, otherwise not
        CHANGELOG

        Added 05.12.2018

        :param config:
        """
        self.config = self.DEFAULT_CONFIG.copy()
        self.config.update(config)

    def __call__(self, areas):

        # First we need to decide which areas will be put in a group with each other. This we will do, by computing
        # Whether to group for each pair:
        groups = self.group_areas(areas)
        combined_areas = []
        for group in groups:
            combined_area = self.combine_areas(group)
            combined_areas.append(combined_area)

        return combined_areas

    def group_areas(self, areas):
        group_membership = defaultdict(list)

        for i in range(len(areas)):

            for j in range(i + 1, len(areas)):
                area_i = areas[i]
                area_j = areas[j]

                # Calculating the distance and the size and using that as parameters to the weight function
                distance = self.area_distance(area_i, area_j)
                size = self.area_size(area_j) + self.area_size(area_j)
                weight = self.config['weight_function'](distance, size)

                if weight < self.config['threshold']:
                    group = group_membership[area_j] + group_membership[area_i] + [area_i, area_j]
                    # removing duplicates from that list
                    group = list(set(group))
                    group_membership[area_j] = group
                    group_membership[area_i] = group

                else:
                    group_membership[area_i].append(area_i)
                    group_membership[area_j].append(area_j)

        groups = []
        for group in list(group_membership.values()):
            if group not in groups:
                groups.append(group)

        return groups

    @staticmethod
    def combine_areas(areas):

        x_min = math.inf
        x_max = 0
        y_min = math.inf
        y_max = 0

        for area in areas:
            x_start = area[0][0]
            y_start = area[1][0]

            x_end = x_start + (area[0][1] - area[0][0])
            y_end = y_start + (area[1][1] - area[1][0])

            if x_start < x_min:
                x_min = x_start
            if y_start < y_min:
                y_min = y_start
            if x_end > x_max:
                x_max = x_end
            if y_end > y_max:
                y_max = y_end

        return (x_min, x_max), (y_min, y_max)

    @staticmethod
    def area_size(area):
        width = area[0][1] - area[0][0]
        height = area[1][1] - area[1][0]
        return width * height

    @staticmethod
    def area_distance(area1, area2):
        # First we need the center point of each area
        area1_center = SimpleAreaGroupingEngine.area_center(area1)
        area2_center = SimpleAreaGroupingEngine.area_center(area2)

        # Calculating the distance between two 2 dimensional points
        distance = math.sqrt((area1_center[0] - area2_center[0])**2 + (area1_center[1] - area2_center[1])**2)
        return distance

    @staticmethod
    def area_center(area):
        center = (area[0][0] + (area[0][1] - area[0][0]) / 2, area[1][0] + (area[1][1] - area[1][0]) / 2)
        return center



import numpy as np


def average_2d(array, area=None):
    """
    Given a 2 dimensional array this function will calculate and return the average of all the elements. The additional
    area argument can be used to specifiy a sub range of indices to be used for the average.
    The area parameter has to be a tuple of the form:
    ( (axis0_start_index, axis0_end_index), (axis1_start_index, axis1_end_index) )

    CHANGELOG

    Added 16.11.2018

    @param np.ndarray array:
    @param tuple area:
    @return:
    """
    if isinstance(area, tuple):
        x_sequence, y_sequence = area
    elif area is None:
        x_sequence = (0, array.shape[0] - 1)
        y_sequence = (0, array.shape[1] - 1)
    else:
        raise TypeError("area input of type {} not supported".format(type(area)))

    # Iterating over all the specified elements, calculating the sum, while also counting the elements,
    # so that later on the average can be calculated as the sum divided by the amount
    summation = 0
    amount = 0

    it = np.nditer(array, flags=['multi_index'])
    while not it.finished:
        y_index, x_index = it.multi_index

        if x_sequence[0] <= x_index <= x_sequence[1] and y_sequence[0] <= y_index <= y_sequence[1]:
            summation += it[0]
            amount += 1

        it.iternext()

    return summation / amount


def threshold_sequencing(array, threshold):
    """
    Given an 1 dimensional array and a threshold value, this function will iterate the array and search sub sequences,
    where the value of array element surpasses the threshold. A list with tuples will be returned, where a tuple
    consists of two indices for the array, the first being the index, where the value first exceeded the threshold and
    the second index being where the value dropped beneath the thr again.

    CHANGELOG

    Added 16.11.2018

    @param np.ndarray array:
    @param float threshold:
    @return:
    """
    sequences = []

    sequence_start = None
    it = np.nditer(array, flags=['multi_index'])
    while not it.finished:

        if sequence_start is None:
            # This means no value was bigger than the threshold yet, so no sequence has started
            # We are only searching for a value bigger than the threshold.
            if it[0] >= threshold:
                sequence_start = it.multi_index[0]

        else:
            # This means we are in the middle of a sequence. When we find a single value
            # that is smaller than the threshold, the sequence ends
            if it[0] < threshold:
                # Saving the sequence with the end being the current index
                sequences.append((sequence_start, it.multi_index[0]))
                sequence_start = None

        it.iternext()

    return sequences


def combinations_2d(iterable1, iterable2):
    """
    Given two lists, a list with all possible combinations of elements from the two lists will be returned. The
    combinations will be in the form of tuples, where the first element is always from the list that was given as
    the first argument to this function.

    CHANGELOG

    Added 16.11.2018

    @param list iterable1:
    @param list iterable2:
    @return:
    """
    combinations = []

    for i in iterable1:
        for j in iterable2:
            combinations.append((i, j))

    return combinations



from setuptools import setup


setup(
    name='lightnimage',
    version='0.0.0.12',
    description='A library to analyze images of lightning strikes',
    url='https://github.com/the16thpythonist/lightnimage',
    author='Jonas Teufel',
    author_email='jonseb1998@gmail.com',
    license='MIT',
    packages=[
        'lightnimage'
    ],
    install_requires=[
        'numpy',
        'scipy',
        'matplotlib',
        'jupyter',
        'pillow',
        'imageio'
    ],
    python_requires='~=3.5',
    zip_safe=False
)
